from ..utils import Scraper
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import requests


class UTSGAthletics:

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('UTSGAthletics initialized.')
        Scraper.logger.info('Not implemented.')
        Scraper.logger.info('UTSGAthletics completed.')
